# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .analytic_type_id import AnalyticTypeId
from .object import Object

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Analytic(pydantic.BaseModel):
    """
    The Analytic object contains details about the analytic technique used to analyze and derive insights from the data or information that led to the creation of a finding or conclusion.
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The analytic category.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the analytic that generated the finding.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the analytic that generated the finding.
    """

    related_analytics: typing.Optional[typing.List[Object]] = pydantic.Field(default=None)
    """
    Other analytics related to this analytic.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The analytic type.
    """

    type_id: AnalyticTypeId = pydantic.Field()
    """
    The analytic type ID.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the analytic that generated the finding.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The analytic version. For example: <code>1.1</code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class HclAppScanOnCloudUrl(str, enum.Enum):
    CLOUD_APPSCAN_COM = "https://cloud.appscan.com"
    EU_CLOUD_APPSCAN_COM = "https://eu.cloud.appscan.com"

    def visit(
        self, cloud_appscan_com: typing.Callable[[], T_Result], eu_cloud_appscan_com: typing.Callable[[], T_Result]
    ) -> T_Result:
        if self is HclAppScanOnCloudUrl.CLOUD_APPSCAN_COM:
            return cloud_appscan_com()
        if self is HclAppScanOnCloudUrl.EU_CLOUD_APPSCAN_COM:
            return eu_cloud_appscan_com()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AppSecApplicationUidType(str, enum.Enum):
    PROVIDER_RELEASE = "provider_release"
    PROVIDER_APPLICATION = "provider_application"

    def visit(
        self, provider_release: typing.Callable[[], T_Result], provider_application: typing.Callable[[], T_Result]
    ) -> T_Result:
        if self is AppSecApplicationUidType.PROVIDER_RELEASE:
            return provider_release()
        if self is AppSecApplicationUidType.PROVIDER_APPLICATION:
            return provider_application()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.url_string import UrlString
from .d_3_fend import D3Fend

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Mitigation(pydantic.BaseModel):
    """
    The MITRE Mitigation object describes the ATT&CK® or ATLAS™ Mitigation ID and/or name that is associated to an attack.
    """

    countermeasures: typing.Optional[typing.List[D3Fend]] = pydantic.Field(default=None)
    """
    The D3FEND countermeasures that are associated with the attack technique. For example: ATT&CK Technique <code>T1003</code> is addressed by Mitigation <code>M1027</code>, and D3FEND Technique <code>D3-OTP</code>.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Mitigation name that is associated with the attack technique. For example: <code>Password Policies</code>, or <code>Code Signing</code>.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The versioned permalink of the Mitigation. For example: <code>https://attack.mitre.org/versions/v14/mitigations/M1027</code>.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Mitigation ID that is associated with the attack technique. For example: <code>M1027</code>, or <code>AML.M0013</code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .....v_1_5_0.resources.applicationsecurityposturefinding.resources.classes.types.application_security_posture_finding import (
    ApplicationSecurityPostureFinding as resources_ocsf_resources_v_1_5_0_resources_applicationsecurityposturefinding_resources_classes_types_application_security_posture_finding_ApplicationSecurityPostureFinding,
)

ApplicationSecurityPostureFinding = resources_ocsf_resources_v_1_5_0_resources_applicationsecurityposturefinding_resources_classes_types_application_security_posture_finding_ApplicationSecurityPostureFinding



# This file was auto-generated by Fern from our API Definition.

from .app_sec_application import AppSecApplication
from .app_sec_application_uid_type import AppSecApplicationUidType
from .app_sec_finding_result import AppSecFindingResult
from .app_sec_get_application_finding_details_response import AppSecGetApplicationFindingDetailsResponse
from .app_sec_get_application_finding_details_response_generic import AppSecGetApplicationFindingDetailsResponseGeneric
from .app_sec_posture_finding import AppSecPostureFinding
from .app_sec_query_application_findings_response import AppSecQueryApplicationFindingsResponse
from .app_sec_query_application_findings_response_generic import AppSecQueryApplicationFindingsResponseGeneric
from .app_sec_query_applications_response import AppSecQueryApplicationsResponse
from .app_sec_query_applications_response_generic import AppSecQueryApplicationsResponseGeneric
from .app_sec_query_findings_response import AppSecQueryFindingsResponse
from .app_sec_query_findings_response_generic import AppSecQueryFindingsResponseGeneric
from .application_id import ApplicationId
from .finding_id import FindingId

__all__ = [
    "AppSecApplication",
    "AppSecApplicationUidType",
    "AppSecFindingResult",
    "AppSecGetApplicationFindingDetailsResponse",
    "AppSecGetApplicationFindingDetailsResponseGeneric",
    "AppSecPostureFinding",
    "AppSecQueryApplicationFindingsResponse",
    "AppSecQueryApplicationFindingsResponseGeneric",
    "AppSecQueryApplicationsResponse",
    "AppSecQueryApplicationsResponseGeneric",
    "AppSecQueryFindingsResponse",
    "AppSecQueryFindingsResponseGeneric",
    "ApplicationId",
    "FindingId",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Node(pydantic.BaseModel):
    """
    Represents a node or a vertex in a graph structure.
    """

    data: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    Additional data about the node stored as key-value pairs. Can include custom properties specific to the node.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable description of the node's purpose or meaning in the graph.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable name or label for the node. Should be descriptive and unique within the graph context.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Categorizes the node into a specific class or type. Useful for grouping and filtering nodes.
    """

    uid: str = pydantic.Field()
    """
    A unique string or numeric identifier that distinguishes this node from all others in the graph. Must be unique across all nodes.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .open_text_core_application_security_credential import OpenTextCoreApplicationSecurityCredential
from .open_text_core_application_security_url import OpenTextCoreApplicationSecurityUrl

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AppsecOpenTextCoreApplicationSecurity(pydantic.BaseModel):
    """
    Configuration for OpenText Core Application Security (formerly Fortify On Demand) as an application security provider.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/opentext-core-applicationsecurity-appsec-setup)
    """

    credential: OpenTextCoreApplicationSecurityCredential = pydantic.Field()
    """
    Credentials used for accessing the OpenText Core Application Security API.
    """

    url: OpenTextCoreApplicationSecurityUrl = pydantic.Field()
    """
    Base URL for the OpenText Core Application Security API. This URL should be the same as the URL used to access the OpenText Core Application Security web interface.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .webhook_event import WebhookEvent

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class WebhookItem(pydantic.BaseModel):
    webhook_url: str = pydantic.Field()
    """
    Webhook URL. Events from providers will be sent to this URL.
    """

    event_filter: typing.Optional[typing.List[WebhookEvent]] = pydantic.Field(default=None)
    """
    If specified, only events matching this list will be sent to `webhook_url`. If no filters are specified, all events sent from providers will be forwarded to `webhook_url`.
    """

    integrator_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    The key used to sign outgoing web hook payloads. If not specified a random key is automatically generated.
    Use this key at the target URL to validate that the incoming payload was signed by Synqly. The payload is signed symmetrically with the `integrator_key` using the `HMAC-SHA256` signature scheme. The key should be randomly generated and between 24 bytes (192 bits) and 64 bytes (512 bits) long.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .edge import Edge
from .graph_query_language_id import GraphQueryLanguageId
from .node import Node

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Graph(pydantic.BaseModel):
    """
    A graph data structure representation with nodes and edges.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The graph description - provides additional details about the graph's purpose and contents.
    """

    edges: typing.Optional[typing.List[Edge]] = pydantic.Field(default=None)
    """
    The edges/connections between nodes in the graph - contains the collection of <code>edge</code> objects defining relationships between nodes.
    """

    is_directed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the graph is directed (<code>true</code>) or undirected (<code>false</code>).
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The graph name - a human readable identifier for the graph.
    """

    nodes: typing.List[Node] = pydantic.Field()
    """
    The nodes/vertices of the graph - contains the collection of <code>node</code> objects that make up the graph.
    """

    query_language: typing.Optional[str] = pydantic.Field(default=None)
    """
    The graph query language, normalized to the caption of the <code>query_language_id</code> value.
    """

    query_language_id: typing.Optional[GraphQueryLanguageId] = pydantic.Field(default=None)
    """
    The normalized identifier of a graph query language that can be used to interact with the graph.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The graph type. Typically useful to represent the specifc type of graph that is used.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier of the graph - a unique ID to reference this specific graph.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.timestamp import Timestamp
from ...base.types.url_string import UrlString
from .product import Product

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class TransformationInfo(pydantic.BaseModel):
    """
    The transformation_info object represents the mapping or transformation used.
    """

    lang: typing.Optional[str] = pydantic.Field(default=None)
    """
    The transformation language used to transform the data.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the transformation or mapping.
    """

    product: typing.Optional[Product] = pydantic.Field(default=None)
    """
    The product or instance used to make the transformation
    """

    time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    Time of the transformation.
    """

    time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Time of the transformation.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the mapping or transformation.
    """

    url_string: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The Uniform Resource Locator String where the mapping or transformation exists.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .fingerprint import Fingerprint
from .software_component_relationship_id import SoftwareComponentRelationshipId
from .software_component_type_id import SoftwareComponentTypeId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SoftwareComponent(pydantic.BaseModel):
    """
    The Software Component object describes characteristics of a software component within a software package.
    """

    author: typing.Optional[str] = pydantic.Field(default=None)
    """
    The author(s) who published the software component.
    """

    hash: typing.Optional[Fingerprint] = pydantic.Field(default=None)
    """
    Cryptographic hash to identify the binary instance of a software component.
    """

    license: typing.Optional[str] = pydantic.Field(default=None)
    """
    The software license applied to this component.
    """

    name: str = pydantic.Field()
    """
    The software component name.
    """

    purl: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Package URL (PURL) to identify the software component. This is a URL that uniquely identifies the component, including the component's name, version, and type. The URL is used to locate and retrieve the component's metadata and content.
    """

    related_component: typing.Optional[str] = pydantic.Field(default=None)
    """
    The package URL (PURL) of the component that this software component has a relationship with.
    """

    relationship: typing.Optional[str] = pydantic.Field(default=None)
    """
    The relationship between two software components, normalized to the caption of the <code>relationship_id</code> value. In the case of 'Other', it is defined by the source.
    """

    relationship_id: typing.Optional[SoftwareComponentRelationshipId] = pydantic.Field(default=None)
    """
    The normalized identifier of the relationship between two software components.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of software component, normalized to the caption of the <code>type_id</code> value. In the case of 'Other', it is defined by the source.
    """

    type_id: typing.Optional[SoftwareComponentTypeId] = pydantic.Field(default=None)
    """
    The type of software component.
    """

    version: str = pydantic.Field()
    """
    The software component version.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .custom_field_mapping import CustomFieldMapping

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class TicketingMock(pydantic.BaseModel):
    """
    Configuration for the Synqly mock in-memory ticketing Provider. This provider is for testing purposes only. It retains tickets for a limited time and does not persist them for long-term usage.
    """

    custom_field_mappings: typing.Optional[typing.List[CustomFieldMapping]] = pydantic.Field(default=None)
    """
    Custom field mappings for this provider.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional name of the mock provider. This value is unused.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .vendor_attributes_severity_id import VendorAttributesSeverityId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class VendorAttributes(pydantic.BaseModel):
    """
    The Vendor Attributes object can be used to represent values of attributes populated by the Vendor/Finding Provider. It can help distinguish between the vendor-prodvided values and consumer-updated values, of key attributes like <code>severity_id</code>.<br>The original finding producer should not populate this object. It should be populated by consuming systems that support data mutability.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The finding severity, as reported by the Vendor (Finding Provider). The value should be normalized to the caption of the <code>severity_id</code> value. In the case of 'Other', it is defined by the source.
    """

    severity_id: typing.Optional[VendorAttributesSeverityId] = pydantic.Field(default=None)
    """
    The finding severity ID, as reported by the Vendor (Finding Provider).
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...common.types.id import Id
from ...engine.types.object import Object

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Agent(pydantic.BaseModel):
    """
    An agent is a human that can be notified when certain conditions are met
    """

    uid: Id
    full_name: str
    email_addr: typing.Optional[str] = None
    phone_number: typing.Optional[str] = None
    time_zone: typing.Optional[str] = None
    job_title: typing.Optional[str] = None
    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the comment schema. The names and values of those attributes are specific to the provider.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.ip_address import IpAddress
from .http_header import HttpHeader
from .http_request_http_method import HttpRequestHttpMethod
from .url import Url

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class HttpRequest(pydantic.BaseModel):
    """
    The HTTP Request object represents the attributes of a request made to a web server. It encapsulates the details and metadata associated with an HTTP request, including the request method, headers, URL, query parameters, body content, and other relevant information.
    """

    args: typing.Optional[str] = pydantic.Field(default=None)
    """
    The arguments sent along with the HTTP request.
    """

    http_headers: typing.Optional[typing.List[HttpHeader]] = pydantic.Field(default=None)
    """
    Additional HTTP headers of an HTTP request or response.
    """

    http_method: typing.Optional[HttpRequestHttpMethod] = pydantic.Field(default=None)
    """
    The <a target='_blank' href='https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods'>HTTP request method</a> indicates the desired action to be performed for a given resource.
    """

    length: typing.Optional[int] = pydantic.Field(default=None)
    """
    The HTTP request length, in number of bytes.
    """

    referrer: typing.Optional[str] = pydantic.Field(default=None)
    """
    The request header that identifies the address of the previous web page, which is linked to the current web page or resource being requested.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the http request.
    """

    url: typing.Optional[Url] = pydantic.Field(default=None)
    """
    The URL object that pertains to the request.
    """

    user_agent: typing.Optional[str] = pydantic.Field(default=None)
    """
    The request header that identifies the operating system and web browser.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Hypertext Transfer Protocol (HTTP) version.
    """

    x_forwarded_for: typing.Optional[typing.List[IpAddress]] = pydantic.Field(default=None)
    """
    The X-Forwarded-For header identifying the originating IP address(es) of a client connecting to a web server through an HTTP proxy or a load balancer.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ActivityId,
    CategoryUid,
    ClassUid,
    ConfidenceId,
    ImpactId,
    IncidentFinding,
    PriorityId,
    SeverityId,
    StatusId,
    TypeUid,
    VerdictId,
)

__all__ = [
    "ActivityId",
    "CategoryUid",
    "ClassUid",
    "ConfidenceId",
    "ImpactId",
    "IncidentFinding",
    "PriorityId",
    "SeverityId",
    "StatusId",
    "TypeUid",
    "VerdictId",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class KeyboardInfo(pydantic.BaseModel):
    """
    The Keyboard Information object contains details and attributes related to a computer or device keyboard. It encompasses information that describes the characteristics, capabilities, and configuration of the keyboard.
    """

    function_keys: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of function keys on client keyboard.
    """

    ime: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Input Method Editor (IME) file name.
    """

    keyboard_layout: typing.Optional[str] = pydantic.Field(default=None)
    """
    The keyboard locale identifier name (e.g., en-US).
    """

    keyboard_subtype: typing.Optional[int] = pydantic.Field(default=None)
    """
    The keyboard numeric code.
    """

    keyboard_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The keyboard type (e.g., xt, ico).
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .hcl_app_scan_on_cloud_credential import HclAppScanOnCloudCredential
from .hcl_app_scan_on_cloud_url import HclAppScanOnCloudUrl

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AppsecHclAppScanOnCloud(pydantic.BaseModel):
    """
    Configuration for HCL AppScan on Cloud as an application security provider.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/hcl-appscan-appsec-setup)
    """

    credential: HclAppScanOnCloudCredential = pydantic.Field()
    """
    Credentials used for accessing the HCL AppScan on Cloud API.
    """

    url: HclAppScanOnCloudUrl = pydantic.Field()
    """
    Base URL for the HCL AppScan on Cloud API. This URL should be the same as the URL used to access the HCL AppScan on Cloud web interface.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .activity_id import ActivityId
from .auth_protocol_id import AuthProtocolId
from .authentication import Authentication
from .category_uid import CategoryUid
from .class_uid import ClassUid
from .logon_type_id import LogonTypeId
from .severity_id import SeverityId
from .status_id import StatusId
from .type_uid import TypeUid

__all__ = [
    "ActivityId",
    "AuthProtocolId",
    "Authentication",
    "CategoryUid",
    "ClassUid",
    "LogonTypeId",
    "SeverityId",
    "StatusId",
    "TypeUid",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...engine.types.object import Object
from ...ocsf.resources.v_1_5_0.resources.applicationsecurityposturefinding.resources.objects.types.application import (
    Application,
)
from .app_sec_application_uid_type import AppSecApplicationUidType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AppSecApplication(Application):
    """
    An application describes the details for an inventoried application as reported by an Application Security tool or other Developer-centric tooling. Applications can be defined as Github repositories or other code-centric resources.
    """

    uid_type: typing.Optional[AppSecApplicationUidType] = pydantic.Field(default=None)
    """
    An attribute that describes the type of the application's unique identifier as defined by the provider. This may be used to help find the application in the provider's user interface. Note that the uid attribute is used as the primary identifier for the application for the purposes of the Synqly API.
    """

    uid_alt_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    An attribute that describes the type of the application's alternate unique identifier as defined by the provider. This may be used to help find the application in the provider's user interface.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the application schema. The names and values of those attributes are specific to the provider.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ..common.errors.bad_gateway_error import BadGatewayError
from ..common.errors.bad_request_error import BadRequestError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.gateway_timeout_error import GatewayTimeoutError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.not_implemented_error import NotImplementedError
from ..common.errors.service_unavailable_error import ServiceUnavailableError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.types.problem import Problem
from ..events.types.event import Event

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SinkClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def post_events(
        self,
        *,
        location: typing.Optional[str] = None,
        request: typing.Sequence[Event],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Writes a batch of `Event` objects to the Sink configured with the token used for authentication.

        Parameters:
            - location: typing.Optional[str]. Updates the path or queue name for the sink. If not provided, the default path or queue name is used. When provided, the path or queue name is appended to the default path or queue name.

            - request: typing.Sequence[Event].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sink/events"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "location": location,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSinkClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def post_events(
        self,
        *,
        location: typing.Optional[str] = None,
        request: typing.Sequence[Event],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Writes a batch of `Event` objects to the Sink configured with the token used for authentication.

        Parameters:
            - location: typing.Optional[str]. Updates the path or queue name for the sink. If not provided, the default path or queue name is used. When provided, the path or queue name is appended to the default path or queue name.

            - request: typing.Sequence[Event].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sink/events"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "location": location,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .d_3_f_tactic import D3FTactic
from .d_3_f_technique import D3FTechnique

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class D3Fend(pydantic.BaseModel):
    """
    The MITRE D3FEND™ object describes the tactic & technique associated with a countermeasure.
    """

    d_3_f_tactic: typing.Optional[D3FTactic] = pydantic.Field(alias="d3f_tactic", default=None)
    """
    The Tactic object describes the tactic ID and/or name that is associated with a countermeasure.
    """

    d_3_f_technique: typing.Optional[D3FTechnique] = pydantic.Field(alias="d3f_technique", default=None)
    """
    The Technique object describes the technique ID and/or name associated with a countermeasure.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The D3FEND™ Matrix version.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ActionId,
    ActivityId,
    CategoryUid,
    ClassUid,
    ConfidenceId,
    DetectionFinding,
    DispositionId,
    ImpactId,
    RiskLevelId,
    SeverityId,
    StatusId,
    TypeUid,
)

__all__ = [
    "ActionId",
    "ActivityId",
    "CategoryUid",
    "ClassUid",
    "ConfidenceId",
    "DetectionFinding",
    "DispositionId",
    "ImpactId",
    "RiskLevelId",
    "SeverityId",
    "StatusId",
    "TypeUid",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Trait(pydantic.BaseModel):
    """
    Describes a characteristic or feature of an entity that was observed. For example, this object can be used to represent specific characteristics derived from events or findings that can be surfaced as distinguishing traits of the entity in question.
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The high-level grouping or classification this trait belongs to.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the trait.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the trait. For example, this can be used to indicate if the trait acts as a contributing factor (increases risk/severity) or a mitigating factor (decreases risk/severity), in the context of the related finding.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the trait.
    """

    values: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The values of the trait.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class OpenTextCoreApplicationSecurityUrl(str, enum.Enum):
    AMS_FORTIFY_COM = "https://api.ams.fortify.com"
    EMEA_FORTIFY_COM = "https://api.emea.fortify.com"
    APAC_FORTIFY_COM = "https://api.apac.fortify.com"
    SGP_FORTIFY_COM = "https://api.sgp.fortify.com"
    FED_FORTIFYGOV_COM = "https://api.fed.fortifygov.com"
    TRIAL_FORTIFY_COM = "https://api.trial.fortify.com"

    def visit(
        self,
        ams_fortify_com: typing.Callable[[], T_Result],
        emea_fortify_com: typing.Callable[[], T_Result],
        apac_fortify_com: typing.Callable[[], T_Result],
        sgp_fortify_com: typing.Callable[[], T_Result],
        fed_fortifygov_com: typing.Callable[[], T_Result],
        trial_fortify_com: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is OpenTextCoreApplicationSecurityUrl.AMS_FORTIFY_COM:
            return ams_fortify_com()
        if self is OpenTextCoreApplicationSecurityUrl.EMEA_FORTIFY_COM:
            return emea_fortify_com()
        if self is OpenTextCoreApplicationSecurityUrl.APAC_FORTIFY_COM:
            return apac_fortify_com()
        if self is OpenTextCoreApplicationSecurityUrl.SGP_FORTIFY_COM:
            return sgp_fortify_com()
        if self is OpenTextCoreApplicationSecurityUrl.FED_FORTIFYGOV_COM:
            return fed_fortifygov_com()
        if self is OpenTextCoreApplicationSecurityUrl.TRIAL_FORTIFY_COM:
            return trial_fortify_com()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Edge(pydantic.BaseModel):
    """
    Represents a connection or relationship between two nodes in a graph.
    """

    data: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    Additional data about the edge such as weight, distance, or custom properties.
    """

    is_directed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the edge is (<code>true</code>) or undirected (<code>false</code>).
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The human-readable name or label for the edge.
    """

    relation: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of relationship between nodes (e.g. is-attached-to , depends-on, etc).
    """

    source: str = pydantic.Field()
    """
    The unique identifier of the node where the edge originates.
    """

    target: str = pydantic.Field()
    """
    The unique identifier of the node where the edge terminates.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier of the edge.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
